//-----------------------------------------------------------------------------
//
// Author: John Clayton
// Date  : Aug.  24, 2001
// Update: Aug.  24, 2001 Obtained this file from "build_8" project.
// Update: Aug.  24, 2001 Added instantiation of "xr16_risc"
// Update: Sept.  4, 2001 Changed memory (16-bit "A" and 16-bit "B")
//
// Description
//-----------------------------------------------------------------------------
// This targets an XC2S200 board which was created for educational purposes.
//
// There are:
//    8  LEDs (led[7:0])
//    4  switches (switch[3:0])
//    1  clock of 32.000 MHz clock, present on GCLK1
//    1  clock of 49.152 MHz clock, present on GCLK0
//    4  lines of ps2 clock input (port A in documentation notes)
//    4  lines of ps2 data input (port A in documentation notes)
//    16 lines of LCD panel control (port B in documentation notes)
//    2  lines of rs232 serial connection (port C in documentation notes)
//-----------------------------------------------------------------------------

// NOTE: This build is for testing out a "xr16_risc" microcontroller.
//
//       These rams are configured for "little endian" transformation.
//
//       That is:  Sequential bytes, 00, 01, 02, 03 on the s8 side emerge
//                 on the s16 side:  01000302.
//                 on the s32 side:  03020100.
//
//                 Sequential words, 0100, 0302 on the s16 side emerge
//                 on the s32 side:  03020100.
//
//       This is called "little endian".  It can be quite confusing at times!

`resetall
`timescale 1ns/100ps

module top (
  sys_clk_0,
  sys_clk_1,
  switch,
  led,
  ps2_clk,
  ps2_data,
  lcd_drive,
  rs232_rxd,
  rs232_txd,
  dat_o,
  debug_o
  );

output [15:0] dat_o;      // for debug only
output [13:0] debug_o;    // for debug only

// I/O declarations
input sys_clk_0;      // 49.152 MHz
input sys_clk_1;      // 32.000 MHz
input [3:0] switch;
input rs232_rxd;

inout [3:0] ps2_clk;
inout [3:0] ps2_data;

output [7:0] led;

output [15:0] lcd_drive;
output rs232_txd;

// Internal signal declarations

reg [9:0] crosshair_x_pos;
reg [9:0] crosshair_y_pos;

wire [7:0] lcd_control;

reg clk_s;               // 49.152/2 MHz = 24.576 MHz

     // Signals from xr16 risc processor
//wire [15:0] xr16_dat;    // temporary, until driver problem is solved.
wire [15:0] xr16_adr;
wire        xr16_word;
wire        xr16_stb;
wire        xr16_dma;
wire        xr16_we;

     // Signals from rs232_syscon
wire [15:0] adr;
wire [15:0] dat;
wire we;
wire stb;
wire rst;
wire master_br;

    // Signals for the bus_rdy generator
reg  [5:0] step_count;
wire [5:0] clocks_to_step;
wire bus_rdy;
wire step;
wire [1:0] processor_control;
wire run_free;
wire forced_reset;

    // Address decode signals
wire        selio;      // High for access to I/O space
wire [7:0]  sel;        // 1 of 8 is active high for I/O space accesses

    // RAM signals
wire ram_sel_1;
wire ram_sel_2;
wire [15:0] ram_1_dat_o;
wire [15:0] ram_2_dat_o;

    // Other...
wire reset = switch[0];  // Simply a renaming exercise

//--------------------------------------------------------------------------
// Clock generation
//--------------------------------------------------------------------------

// This uses up an additional GCLK resource.
always @(posedge sys_clk_0)
begin
  clk_s <= ~clk_s;
end

//--------------------------------------------------------------------------
// Instantiations
//--------------------------------------------------------------------------

lcd_test_pongball_crosshair lcd_block (
  .sys_clk(sys_clk_1),
  .lcd_clk(sys_clk_0),
  .sys_reset(lcd_control[1]),
  .lcd_reset(lcd_control[0]),
  .ball_x_enable(lcd_control[2]),
  .ball_y_enable(lcd_control[3] && switch[3]),  // switch[3] not needed!
  .crosshair_x_pos(crosshair_x_pos),
  .crosshair_y_pos(crosshair_y_pos),
  .crosshair_color(lcd_control[6:4]),
  .lcd_drive(lcd_drive)
  );


xr16 risc1 (
            .clk(clk_s),
            .rst(rst || forced_reset),
            .rdy( (bus_rdy || run_free) && ~(stb && ~xr16_stb) ),
            .int_req(1'b0),
            .dma_req(1'b0),
            .zerodma(1'b0),
            .insn(dat),
            .word(xr16_word),
            .write(xr16_we),
            .stb_o(xr16_stb),
            .dma(xr16_dma),
            .a(xr16_adr),
//            .d(xr16_dat)
            .d(dat)
            );



rs232_syscon #(
               4,             // Number of Hex digits for addresses.
               4,             // Number of Hex digits for data.
               2,             // Number of Hex digits for quantity.
               16,            // Characters in the input buffer
               4,             // Bits in the buffer pointer
               255,           // Clocks before watchdog timer expires
               8,             // Bits in watchdog timer
               8,             // Number of data fields displayed per line
               3,             // Number of bits in the fields counter
               2              // Number of bits in the digits counter
               )
  syscon_1 (                  // instance name
  .clk_i(clk_s),
  .reset_i(reset),
  .master_bg_i(master_br),
  .ack_i(switch[1]),
  .err_i(switch[2]),
  .master_adr_i(xr16_adr),
  .master_stb_i(xr16_stb),
  .master_we_i(xr16_we),
  .rs232_rxd_i(rs232_rxd),
  .dat_io(dat),
  .rst_o(rst),
  .master_br_o(master_br),
  .stb_o(stb),
  .cyc_o(),
  .adr_o(adr),
  .we_o(we),
  .rs232_txd_o(rs232_txd)
  );


// I/O decodes
assign      selio   = adr[15:8] == 8'hFF && stb;
assign      sel[0]  = adr[7:5] == 0 && selio;
assign      sel[1]  = adr[7:5] == 1 && selio;
assign      sel[2]  = adr[7:5] == 2 && selio;
assign      sel[3]  = adr[7:5] == 3 && selio;
assign      sel[4]  = adr[7:5] == 4 && selio;
assign      sel[5]  = adr[7:5] == 5 && selio;
assign      sel[6]  = adr[7:5] == 6 && selio;
assign      sel[7]  = adr[7:5] == 7 && selio;


reg_8_pack #(
             8,               // Size of r0
             1,               // Size of r1
             1,               // Size of r2
             1,               // Size of r3
             8,               // Size of r4
             10,              // Size of r5
             10,              // Size of r6
             8,               // Size of r7
             1,               // Read only regs.
             16               // Size of the data bus.
             )
  reg_8_pack_1                // Instance name
  (
   .clk_i(clk_s),
   .rst_i(reset),
   .sel_i(sel[0]),
   .we_i(we),
   .adr_i(adr[3:1]),  // byte addressed... word accessible only
   .dat_io(dat),
   .r0({~ps2_clk,~ps2_data}),
   .r1(),
   .r2(),
   .r3(),
   .r4(led),
   .r5(crosshair_x_pos),
   .r6(crosshair_y_pos),
   .r7(lcd_control)
   );


reg_4_pack_clrset #(
             1,               // Size of r0
             1,               // Size of r1
             6,               // Size of r2
             2,               // Size of r3
             0,               // Read only regs.
             8                // Size of the data bus.
             )
  reg_4_pack_clrset_1         // Instance name
  (
   .clk_i(clk_s),
   .rst_i(reset),
   .sel_i(sel[1]),
   .we_i(we),
   .adr_i(adr[1:0]),  // byte addressed... byte accessible only
   .clr_i({2'b0,step,1'b0}),
   .set_i(4'b0),
   .dat_io(dat[7:0]),
   .r0(),
   .r1(step),
   .r2(clocks_to_step),
   .r3(processor_control)
   );

assign run_free = processor_control[1];
assign forced_reset = processor_control[0];


// This part generates the bus_rdy signal for the processor.
// This is how single stepping is implemented.  Actually, execution in
// steps of 2,3 or more instructions at a time is also allowed.
always @(posedge clk_s)
begin
  if (reset) step_count <= 0;
  else if (step) step_count <= clocks_to_step;
  else if (bus_rdy) step_count <= step_count - 1;
end
assign bus_rdy = (step_count > 0);

// 2k bytes of RAM (1k x 16) It is byte addressed...
assign ram_sel_1 = ((adr[15:11] == 0) && stb);  // 0000 - 07fe hex
assign dat = (ram_sel_1 && ~we)?ram_1_dat_o:{16{1'bZ}};

ramb16_s16_s16 
  dp_ram_block_1 
  (
  .dat_o_s16a(ram_1_dat_o),
  .adr_i_s16a(adr[10:1]),
  .dat_i_s16a(dat),
  .rst_i_s16a(reset),
  .we_i_s16a(we && ram_sel_1),
  .clk_i_s16a(sys_clk_0),
  .dat_o_s16b(),
  .adr_i_s16b(10'b0),
  .dat_i_s16b(16'b0),
  .rst_i_s16b(1'b0),
  .we_i_s16b(1'b0),
  .clk_i_s16b(sys_clk_0)
  );


// 2k bytes of RAM (1k x 16) It is byte addressed
assign ram_sel_2 = ((adr[15:11] == 4) && stb);  // 2000 - 27ff hex
assign dat = (ram_sel_2 && ~we)?ram_2_dat_o:{16{1'bZ}};

ramb16_s16_s16 
  dp_ram_block_2 
  (
  .dat_o_s16a(ram_2_dat_o),
  .adr_i_s16a(adr[10:1]),
  .dat_i_s16a(dat),
  .rst_i_s16a(reset),
  .we_i_s16a(we && ram_sel_2),
  .clk_i_s16a(sys_clk_0),
  .dat_o_s16b(),
  .adr_i_s16b(10'b0),
  .dat_i_s16b(16'b0),
  .rst_i_s16b(1'b0),
  .we_i_s16b(1'b0),
  .clk_i_s16b(sys_clk_0)
  );



//--------------------------------------------------------------------------
// Module code
//--------------------------------------------------------------------------

assign debug_o[0] = bus_rdy;
assign debug_o[1] = stb;
assign debug_o[2] = we;
assign debug_o[3] = xr16_word;
assign debug_o[4] = master_br;
assign debug_o[13:5] = adr[8:0];

assign dat_o = dat;

endmodule

