/* xr16.v -- xr16 pipelined RISC processor synthesizable Verilog model
 *
 * Copyright (C) 1999, 2000, Gray Research LLC.  All rights reserved.
 * The contents of this file are subject to the XSOC License Agreement;
 * you may not use this file except in compliance with this Agreement.
 * See the LICENSE file.
 *
 * $Header: /dist/xsocv/xr16.v 7     4/06/00 10:55a Jan $
 * $Log: /dist/xsocv/xr16.v $
 * 
 * 7     4/06/00 10:55a Jan
 * polish
 */ 


//----------------------------------------------------------------------------
// This is the ctrl.v module of the xr16 RISC processor
//

// Opcodes (IR[15:12])
`define ADD     4'h0
`define SUB     4'h1
`define ADDI    4'h2
`define RR      4'h3
`define RI      4'h4
`define LW      4'h5
`define LB      4'h6
`define SW      4'h8
`define SB      4'h9
`define JAL     4'hA
`define Bcond   4'hB
`define CALL    4'hC
`define IMM     4'hD

// Functions (IR[7:4])
`define AND     4'h0
`define OR      4'h1
`define XOR     4'h2
`define ANDN    4'h3
`define ADC     4'h4
`define SBC     4'h5
`define SRL     4'h6
`define SRA     4'h7
`define SLL     4'h8

// Conditional branches (IR[11:8])
`define BR      4'h0
`define BRN     4'h1
`define BEQ     4'h2
`define BNE     4'h3
`define BC      4'h4
`define BNC     4'h5
`define BV      4'h6
`define BNV     4'h7
`define BLT     4'h8
`define BGE     4'h9
`define BLE     4'hA
`define BGT     4'hB
`define BLTU    4'hC
`define BGEU    4'hD
`define BLEU    4'hE
`define BGTU    4'hF

`define INT     16'hAE01    // int ::= jal r14,10(r0)

module control(
               clk,
               rst,
               rdy,
               int_req,
               dma_req,
               zerodma,
               insn,
               a15,
               z,
               n,
               co,
               v,
               word_nxt,
               write_nxt,
               ldst_nxt,
               dma,
               rf_we,
               rna,
               rnb,
               fwd,
               imm,
               sextimm4,
               zextimm4,
               wordimm4,
               imm12,
               pipe_ce,
               b15_4_ce,
               add,
               ci,
               logicop,
               sri,
               sum_en,
               logic_en,
               shl_en,
               shr_en,
               zeroext_en,
               ret_en,
               branch,
               brdisp,
               selpc,
               zeropc,
               dmapc,
               pc_ce,
               ret_ce
               );

    parameter       W   = 16;   // data word width
    parameter       N   = W-1;  // msb index
    parameter       IW  = 16;   // instruction word width
    parameter       IN  = 15;   // instruction word msb index

    // ports
    input           clk;        // global clock
    input           rst;        // global async reset
    input           rdy;        // current memory access is ready
    input           int_req;    // interrupt request
    input           dma_req;    // DMA request
    input           zerodma;    // zero DMA counter request
    input   [IN:0]  insn;       // new instruction word
    input           a15;        // A operand msb
    input           z;          // zero result condition code
    input           n;          // negative result condition code
    input           co;         // carry-out result codition code
    input           v;          // oVerflow result condition code
    output          word_nxt;   // next access is word wide
    output          write_nxt;  // next access is write
    output          ldst_nxt;   // next access is load/store or dma
    output          dma;        // current access is a DMA transfer
    output          rf_we;      // register file write enable
    output  [3:0]   rna;        // register file port A register number
    output  [3:0]   rnb;        // register file port B register number
    output          fwd;        // forward result bus into A operand register
    output  [11:0]  imm;        // 12-bit immediate field
    output          sextimm4;   // sign-extend 4-bit immediate operand
    output          zextimm4;   // zero-extend 4-bit immediate operand
    output          wordimm4;   // word-offset 4-bit immediate operand
    output          imm12;      // 12-bit immediate operand
    output          pipe_ce;    // pipeline clock enable
    output          b15_4_ce;   // b[15:4] clock enable
    output          add;        // 1 => A + B; 0 => A - B
    output          ci;         // carry-in
    output [1:0]    logicop;    // logic unit opcode
    output          sri;        // shift right msb input
    output          sum_en;     // adder output enable
    output          logic_en;   // logic unit output enable
    output          shl_en;     // shift left output enable
    output          shr_en;     // shift right output enable
    output          zeroext_en; // zero-extension output enable
    output          ret_en;     // return address output enable
    output          branch;     // branch taken
    output [7:0]    brdisp;     // 8-bit branch displacement
    output          selpc;      // address mux selects next PC
    output          zeropc;     // force next PC to 0
    output          dmapc;      // use DMA register in PC register file
    output          pc_ce;      // PC clock enable
    output          ret_ce;     // return address clock enable
    
    reg             add;
    reg             ci;
    reg             branch;
    reg             sum_en;
    reg             logic_en;
    reg             shl_en;
    reg             shr_en;
    reg             zeroext_en;
    reg             ret_en;

    // locals
    reg [IN:0]  if_ir, ir, ex_ir;   // instruction registers

    // IR fields
    wire [3:0]  op      = ir[15:12];    // opcode
    wire [3:0]  rd      = ir[11:8];     // destination register
    wire [3:0]  ra      = ir[7:4];      // source register A
    wire [3:0]  rb      = ir[3:0];      // source register B
    wire [3:0]  cond    = ir[11:8];     // branch condition
    wire [3:0]  fn      = ir[7:4];      // rr or ri function sub-opcode
    wire [3:0]  ex_op   = ex_ir[15:12]; // EX stage opcode
    wire [3:0]  ex_rd   = ex_ir[11:8];  // EX stage destination register
    wire [3:0]  ex_fn   = ex_ir[7:4];   // EX stage function sub-opcode
    assign      imm     = ir[11:0];     // 12-bit immediate field
    assign      brdisp  = ex_ir[7:0];   // 8-bit branch displacement

    // IF stage instruction decoding
    assign imm12    = op==`CALL || op==`IMM;
    assign sextimm4 = op==`ADDI || op==`RI;
    assign zextimm4 = op==`LB || op==`SB;
    assign wordimm4 = op==`LW || op==`SW || op==`JAL;
    wire addsub     = op==`ADD || op==`SUB || op==`ADDI;
    wire call       = op==`CALL;
    wire st         = op==`SW || op==`SB;
    wire rrri       = op==`RR || op == `RI;
    wire adcsbc     = rrri && (fn==`ADC || fn==`SBC);
    wire dcintinh   = op==`ADD || op==`SUB || op==`ADDI || op==`Bcond ||
                      op==`CALL || op==`IMM || adcsbc;
    wire sub        = op==`SUB || (rrri && (fn==`SBC));

    // EX stage decoding
    wire ex_ldst    = ex_op==`LW || ex_op==`LB || ex_op==`SW || ex_op==`SB;
    wire ex_lbsb    = ex_op==`LB || ex_op==`SB;
    wire ex_results = ex_op==`ADD || ex_op==`SUB || ex_op==`ADDI ||
                      ex_op==`RR || ex_op==`RI ||
                      ex_op==`LW || ex_op==`LB || ex_op==`JAL || ex_op==`CALL;
    wire ex_jump    = (ex_op==`JAL || ex_op==`CALL);
    reg  ex_call;   // EX stage call
    reg  ex_st;     // EX stage store

    // memory access FSM states -- each access is an instruction fetch,
    // unless DMA or load/store is pending
    reg  ifetch;    // "current access is insn fetch"
    reg  dma;       // "current access is DMA"
                    // "current access is load/store" ::= ~(ifetch|dma)
    // annul FSM states
    reg  sync_reset; // synchronous memory FSM reset
    reg  dc_annul;  // annul DC stage instruction
    reg  ex_annul;  // annul EX stage instruction

    // interrupt state
    reg  int_pend;  // interrupt is pending
    reg  dc_int;    // DC stage instruction is int
    wire if_int;    // IF stage 'interrupt in progress'

    // DMA state
    reg  dma_pend;  // DMA is pending
    reg  zero_pend; // zero (reset) DMA is pending

    // memory access FSM transitions:
    // case (state)
    //   IF:  state_nxt = dma_pend ? DMA : ldst_pend ? LS : IF;
    //   DMA: state_nxt =                  ldst_pend ? LS : IF;
    //   LS:  state_nxt =                                   IF;
    // endcase
    wire ldst_pend  = ex_ldst & ~ex_annul;
    wire if_nxt     = ifetch&~dma_pend&~ldst_pend | dma&~ldst_pend |
                      ~ifetch&~dma;
    wire dma_nxt    = ifetch&dma_pend;
    wire ldst_nxt   = ifetch&~dma_pend&ldst_pend | dma&ldst_pend;

    wire jump       = ex_jump && ~ex_annul;
    wire exannul_nxt= sync_reset | branch | jump | dc_annul;

    // FSM output decodes
    assign pipe_ce   = rdy&if_nxt;
    assign pc_ce     = rdy&(if_nxt|dma_nxt);
    assign ret_ce    = rdy&if_nxt&~dc_int;
    assign word_nxt  = ~(ldst_nxt&ex_lbsb);
    assign write_nxt = (ldst_nxt&ex_st);
    assign dmapc     = dma_nxt;
    assign selpc     = if_nxt&~jump | dma_nxt;
    assign zeropc    = (zero_pend&dma_nxt) | sync_reset;
    assign if_int    = int_pend&~branch&~jump&~dcintinh;



    // IR clocking
    always @(posedge clk or posedge rst) begin
        if (rst) begin
            if_ir <= 0;
            ir <= 0;
            ex_ir <= 0;
        end
        else begin
            if (ifetch && rdy)
                if_ir <= insn;
            if (pipe_ce) begin
                ir <= (if_int) ? `INT : (ifetch) ? insn : if_ir;
                ex_ir <= ir;
            end
        end
    end

    // instruction decode clocking
    always @(posedge clk or posedge rst) begin
        if (rst) begin
            ex_call <= 0;
            ex_st <= 0;
        end
        else if (pipe_ce) begin
            ex_call <= call;
            ex_st <= st;
        end
    end

    // memory access FSM clocking
    always @(posedge clk or posedge rst) begin
        if (rst) begin
            ifetch <= 1;
            dma <= 0;
        end
        else if (rdy) begin
            ifetch <= if_nxt;
            dma <= dma_nxt;
        end
    end

    // annul states clocking
    always @(posedge clk or posedge rst) begin
        if (rst)
            sync_reset <= 1;
        else if (rdy)
            sync_reset <= 0;
    end
    always @(posedge clk or posedge rst) begin
        if (rst) begin
            dc_annul <= 1;
            ex_annul <= 1;
        end
        else if (pipe_ce) begin
            dc_annul <= sync_reset | branch | jump;
            ex_annul <= exannul_nxt;
        end
    end

    // DMA request clocking
    always @(posedge clk or posedge rst) begin
        if (rst)
            dma_pend <= 0;
        else if (dma_req)
            dma_pend <= 1;
        else if (dma)
            dma_pend <= 0;
    end
    always @(posedge clk or posedge rst) begin
        if (rst)
            zero_pend <= 0;
        else if (zerodma)
            zero_pend <= 1;
        else if (dma)
            zero_pend <= 0;
    end

    // interrupt request clocking
    always @(posedge clk or posedge rst) begin
        if (rst)
            int_pend <= 0;
        else if (int_req)
            int_pend <= 1;
        else if (if_int && pipe_ce)
            int_pend <= 0;
    end
    always @(posedge clk or posedge rst) begin
        if (rst)
            dc_int <= 0;
        else if (pipe_ce)
            dc_int <= if_int;
    end

    // DC stage operand selection
    wire [3:0]  rsrc    = call ? 0 : rrri ? rd : ra;
    wire [3:0]  rdest   = ex_call ? 15 : ex_rd;
    assign      rna     = clk ? rsrc : rdest;
    assign      rnb     = clk ? (st ? rd : rb) : (ex_call ? 15 : ex_rd);
    assign      rf_we   = ex_results & pipe_ce & ~ex_annul & (rdest != 0);
    assign      fwd     = (rsrc==rdest) & (rdest!=0) && ex_results & ~ex_annul;
    assign      b15_4_ce= pipe_ce && ~(ex_op==`IMM && ~ex_annul);

    // DC stage conditional branches
    reg taken;
    always @(posedge clk or posedge rst) begin
        if (rst)
            branch = 0;
        else if (pipe_ce) begin
            case (cond)
            `BR:   taken = 1;
            `BRN:  taken = 0;
            `BEQ:  taken = z;
            `BNE:  taken = ~z;
            `BC:   taken = co;
            `BNC:  taken = ~co;
            `BV:   taken = v;
            `BNV:  taken = ~v;
            `BLT:  taken = n^v;
            `BGE:  taken = ~(n^v);
            `BLE:  taken = (n^v)|z;
            `BGT:  taken = ~((n^v)|z);
            `BLTU: taken = ~z&~co;
            `BGEU: taken = z|co;
            `BLEU: taken = z|~co;
            `BGTU: taken = ~z&co;
            endcase
            branch = op==`Bcond && taken && ~exannul_nxt;
        end
    end

    // EX stage ALU control
    always @(posedge clk or posedge rst) begin
        if (rst) begin
            add <= 1;
            ci <= 0;
        end
        else if (pipe_ce) begin
            add <= ~sub;
            ci <= sub ^ (adcsbc & co);
        end
    end
    assign  logicop = ex_ir[5:4];
    assign  sri     = ex_fn==`SRA && a15;

    // EX stage result multiplexer control
    always @(posedge clk or posedge rst)
    begin
      if (rst)
      begin
        sum_en     <= 0;
        logic_en   <= 0;
        zeroext_en <= 0;
        shr_en     <= 0;
        shl_en     <= 0;
        ret_en     <= 0;
      end
      else if (pipe_ce)
      begin
        sum_en     <= (addsub || adcsbc);
        logic_en   <= (rrri&& (fn==`AND || fn==`OR || fn==`XOR || fn==`ANDN));
        zeroext_en <= (op==`LB);
        shr_en     <= (rrri && (fn==`SRL || fn==`SRA));
        shl_en     <= (rrri && fn==`SLL);
        ret_en     <= (op==`JAL || op==`CALL);
      end
    end

endmodule


//-----------------------------------------------------------------------------
// This is the datapath module of the xr16 RISC processor
//

module datapath(
                clk,
                rst,
                rf_we,
                rna,
                rnb,
                fwd,
                imm,
                sextimm4,
                zextimm4,
                wordimm4,
                imm12,
                pipe_ce,
                b15_4_ce,
                add,
                ci,
                logicop,
                sri,
                sum_en,
                logic_en,
                shl_en,
                shr_en,
                zeroext_en,
                ret_en,
                store_ld,
                store_ud,
                store_udlt,
                branch,
                brdisp,
                selpc,
                zeropc,
                dmapc,
                pc_ce,
                ret_ce,
                a15,
                z,
                n,
                co,
                v,
                addr_nxt,
                res
                );

    parameter       W   = 16;   // word width
    parameter       N   = W-1;  // msb index

    // ports
    input           clk;        // global clock
    input           rst;        // global async reset
    input           rf_we;      // register file write enable
    input [3:0]     rna;        // register file port A register number
    input [3:0]     rnb;        // register file port B register number
    input           fwd;        // forward result bus into A operand reg
    input [11:0]    imm;        // 12-bit immediate field
    input           sextimm4;   // sign-extend 4-bit immediate operand
    input           zextimm4;   // zero-extend 4-bit immediate operand
    input           wordimm4;   // word-offset 4-bit immediate operand
    input           imm12;      // 12-bit immediate operand
    input           pipe_ce;    // pipeline clock enable
    input           b15_4_ce;   // b[15:4] clock enable
    input           add;        // 1 => A + B; 0 => A - B
    input           ci;         // carry-in
    input [1:0]     logicop;    // logic unit opcode
    input           sri;        // shift right msb input
    input           sum_en;     // adder output enable
    input           logic_en;   // logic unit output enable
    input           shl_en;     // shift left output enable
    input           shr_en;     // shift right output enable
    input           zeroext_en; // zero-extension output enable
    input           ret_en;     // return address output enable
    input           store_ud;   // store data MSB output enable
    input           store_ld;   // store data LSB output enable
    input           store_udlt; // store data MSB->LSB output enable
    input           branch;     // branch taken
    input [7:0]     brdisp;     // 8-bit branch displacement
    input           selpc;      // address mux selects next PC
    input           zeropc;     // force next PC to 0
    input           dmapc;      // use DMA register in PC register file
    input           pc_ce;      // PC clock enable
    input           ret_ce;     // return address clock enable
    output          a15;        // A operand msb
    output          z;          // zero result condition code
    output          n;          // negative result condition code
    output          co;         // carry-out result codition code
    output          v;          // oVerflow result condition code
    output  [N:0]   addr_nxt;   // address of next memory access
    inout   [N:0]   res;        // on-chip data bus
    tri     [N:0]   res;

    reg             z, n, co, v;
    reg     [N:0]   addr_nxt;

    // locals
    wire [N:0]  areg_nxt, breg_nxt; // reg file read port RAM output buses
    reg  [N:0]  areg, breg; // register file read ports
    reg  [N:0]  a, b;       // operand registers
    reg  [N:0]  dout;       // store data output register
    reg  [N:0]  sum;        // adder output
    reg  [N:0]  logic;      // logic unit output
    wire [N:0]  pc;         // current PC
    reg  [N:0]  pcnext;     // next PC  // REVIEW
    reg  [N:0]  ret;        // return address (DC stage PC)
    wire [6:0]  brdispext = {brdisp[7],brdisp[7],brdisp[7],brdisp[7],
                                 brdisp[7],brdisp[7],brdisp[7]};

    // submodules
    ram16x16s aregs(.wclk(clk), .addr(rna), .we(rf_we), .d(res), .o(areg_nxt));
    ram16x16s bregs(.wclk(clk), .addr(rnb), .we(rf_we), .d(res), .o(breg_nxt));
    ram16x16s pcs(.wclk(clk), .addr({3'b0,dmapc}), .we(pc_ce), .d(addr_nxt),
                  .o(pc));

    // register file output registers
    always @(negedge clk or posedge rst) begin
        if (rst) begin
            areg <= 0;
            breg <= 0;
        end
        else begin
            areg <= areg_nxt;
            breg <= breg_nxt;
        end
    end

    // operand registers
    always @(posedge clk or posedge rst) begin
        if (rst) begin
            a <= 0;
            b <= 0;
            dout <= 0;
        end
        else begin
            if (pipe_ce) begin
                a <= fwd ? res : areg;
                if (sextimm4 || zextimm4)
                    b[3:0] <= imm[3:0];
                else if (wordimm4)
                    b[3:0] <= {imm[3:1],1'b0};
                else if (imm12)
                    b[3:0] <= 0;
                else
                    b[3:0] <= breg[3:0];
                dout <= breg;
            end
            if (b15_4_ce) begin
                if (sextimm4)
                    b[15:4] <= {12{imm[3]}};
                else if (zextimm4)
                    b[15:4] <= 12'b0;
                else if (wordimm4)
                    b[15:4] <= {11'b0,imm[0]};
                else if (imm12)
                    b[15:4] <= imm[11:0];
                else
                    b[15:4] <= breg[15:4];
            end
        end
    end

    // ALU
    assign a15 = a[15];
    reg ign;
    reg co0;
    always @(a or b or add or ci) begin
        if (add) begin
            {co0,sum,ign} = {a,ci} + {b,1'b1};
            co = co0;
        end
        else begin
            {co0,sum,ign} = {a,ci} - {b,1'b1};
            co = ~co0;
        end
        z = sum == 0;
        n = sum[15];
        // sum[15] = a[15] ^ b[15] ^ c15 <===> c15 = sum[15] ^ a[15] ^ b[15]
        v = co0 ^ sum[15] ^ a[15] ^ b[15];
    end
    always @(a or b or logicop) begin
        case (logicop)
        0: logic <= a&b;
        1: logic <= a|b;
        2: logic <= a^b;
        3: logic <= a&~b;
        endcase
    end

    // address/PC unit
    always @(branch or pc or brdispext or brdisp or dmapc) begin
        if (branch & ~dmapc)
            pcnext <= pc + {brdispext[6:0],brdisp[7:0],1'b0};
        else
            pcnext <= pc + 2;
    end
    always @(sum or pcnext or selpc or zeropc) begin
        if (zeropc)
            addr_nxt <= 0;
        else if (selpc)
            addr_nxt <= pcnext;
        else
            addr_nxt <= sum;
    end
    always @(posedge clk or posedge rst) begin
        if (rst)
            ret <= 0;
        else if (ret_ce)
            ret <= pc;
    end

    // result multiplexer
    assign res       = (sum_en     ) ? sum              : 16'bz;
    assign res       = (logic_en   ) ? logic            : 16'bz;
    assign res       = (shl_en     ) ? { a[14:0],1'b0 } : 16'bz;
    assign res       = (shr_en     ) ? { sri, a[15:1] } : 16'bz;
    assign res       = (ret_en     ) ? ret              : 16'bz;
    assign res[7:0]  = (store_ld   ) ? dout[7:0]        : 8'bz;
    assign res[15:8] = (store_ud   ) ? dout[15:8]       : 8'bz;
    assign res[7:0]  = (store_udlt ) ? dout[15:8]       : 8'bz;
    assign res[15:8] = (zeroext_en ) ? 0                : 8'bz;

endmodule





//-----------------------------------------------------------------------------

module xr16(
            clk,
            rst,
            rdy,
            int_req,
            dma_req,
            zerodma,
            insn,
            word,
            write,
            stb_o,
            dma,
            a,
            d
            );

    parameter       W   = 16;   // word width
    parameter       N   = W-1;  // msb index

    input           clk;        // global clock
    input           rst;        // global async reset
    input           rdy;        // current memory access is ready
    input           int_req;    // interrupt request
    input           dma_req;    // DMA request
    input           zerodma;    // zero DMA counter request
    input   [N:0]   insn;       // new instruction word
    output          word;       // current access is word wide
    output          write;      // current access is write
    output          stb_o;      // current access is load or store
    output          dma;        // current access is a DMA transfer
    output  [N:0]   a;          // address of current memory access
    inout   [N:0]   d;          // data bus

    reg         word;       // Indicates current cycle is word wide
    reg         write;      // Indicates current cycle is write
    reg [N:0]   a;          // Address bus

    
    // locals
    wire [N:0]  addr_nxt;   // Address of the next cycle
    wire [N:0]  res;        // The "result" bus (inout to datapath)
    wire        stb_o;      // Indicates bus cycle is active
    wire        write_nxt;  // next cycle is a write
    wire        word_nxt;   // next cycle is a word wide cycle
    wire        ldst_nxt;   // next cycle is a load or store (includes dma)
    wire        a15;        // A operand msb
    wire        z;          // zero result condition code
    wire        n;          // negative result condition code
    wire        co;         // carry-out result codition code
    wire        v;          // oVerflow result condition code
    wire        rf_we;      // register file write enable
    wire [3:0]  rna;        // register file port A register number
    wire [3:0]  rnb;        // register file port B register number
    wire        fwd;        // forward result bus into A operand register
    wire [11:0] imm;        // 12-bit immediate field
    wire        sextimm4;   // sign-extend 4-bit immediate operand
    wire        zextimm4;   // zero-extend 4-bit immediate operand
    wire        wordimm4;   // word-offset 4-bit immediate operand
    wire        imm12;      // 12-bit immediate operand
    wire        pipe_ce;    // pipeline clock enable
    wire        b15_4_ce;   // b[15:4] clock enable
    wire        add;        // 1 => A + B; 0 => A - B
    wire        ci;         // carry-in
    wire [1:0]  logicop;    // logic unit opcode
    wire        sri;        // shift right msb input
    wire        sum_en;     // adder output enable
    wire        logic_en;   // logic unit output enable
    wire        zeroext_en; // zero-extension output enable
    wire        shr_en;     // shift right output enable
    wire        shl_en;     // shift left output enable
    wire        ret_en;     // return address output enable
    wire        branch;     // branch taken
    wire [7:0]  brdisp;     // 8-bit branch displacement
    wire        selpc;      // address mux selects next PC
    wire        zeropc;     // force next PC to 0
    wire        dmapc;      // use DMA register in PC register file
    wire        pc_ce;      // PC clock enable
    wire        ret_ce;     // return address clock enable
    wire        store_udlt; // enable data upper half to drive lower half
    wire        store_ud;   // drive data upper half
    wire        store_ld;   // drive data lower half

    reg         load_ud;    // read upper half of the data bus
    reg         load_ld;    // read lower half of the data bus

    // submodules
    control ctrl(
                 .clk(clk),
                 .rst(rst),
                 .rdy(rdy),
                 .int_req(int_req),
                 .dma_req(dma_req),
                 .zerodma(zerodma),
                 .insn(insn),
                 .a15(a15),
                 .z(z),
                 .n(n),
                 .co(co),
                 .v(v),
                 .word_nxt(word_nxt),
                 .write_nxt(write_nxt),
                 .ldst_nxt(ldst_nxt),
                 .dma(dma),
                 .rf_we(rf_we),
                 .rna(rna),
                 .rnb(rnb),
                 .fwd(fwd),
                 .imm(imm),
                 .sextimm4(sextimm4),
                 .zextimm4(zextimm4),
                 .wordimm4(wordimm4),
                 .imm12(imm12),
                 .pipe_ce(pipe_ce),
                 .b15_4_ce(b15_4_ce),
                 .add(add),
                 .ci(ci),
                 .logicop(logicop),
                 .sri(sri),
                 .sum_en(sum_en),
                 .logic_en(logic_en),
                 .shl_en(shl_en),
                 .shr_en(shr_en),
                 .zeroext_en(zeroext_en),
                 .ret_en(ret_en),
                 .branch(branch),
                 .brdisp(brdisp),
                 .selpc(selpc),
                 .zeropc(zeropc),
                 .dmapc(dmapc),
                 .pc_ce(pc_ce),
                 .ret_ce(ret_ce)
                 );

    datapath dp(
               .clk(clk),
               .rst(rst),
               .rf_we(rf_we),
               .rna(rna),
               .rnb(rnb),
               .fwd(fwd),
               .imm(imm),
               .sextimm4(sextimm4),
               .zextimm4(zextimm4),
               .wordimm4(wordimm4),
               .imm12(imm12),
               .pipe_ce(pipe_ce),
               .b15_4_ce(b15_4_ce),
               .add(add),
               .ci(ci),
               .logicop(logicop),
               .sri(sri),
               .sum_en(sum_en),
               .logic_en(logic_en),
               .shl_en(shl_en),
               .shr_en(shr_en),
               .zeroext_en(zeroext_en),
               .ret_en(ret_en),
               .store_ld(store_ld),
               .store_ud(store_ud),
               .store_udlt(store_udlt),
               .branch(branch),
               .brdisp(brdisp),
               .selpc(selpc),
               .zeropc(zeropc),
               .dmapc(dmapc),
               .pc_ce(pc_ce),
               .ret_ce(ret_ce),
               .a15(a15),
               .z(z),
               .n(n),
               .co(co),
               .v(v),
               .addr_nxt(addr_nxt),
               .res(res)
               );

// This part decides how the data bus should be driven (low half or high half
// or entire bus), and also generates latched versions of the "word" and
// "read" signals.  The "d" bus is the result bus from the datapath module.
// Therefore, it is used for internal operations.  The signal "ldst_nxt"
// indicates an upcoming load/store operation (including dma transfers),
// which would mean use of the data bus to and from memory/peripherals.

// Also, note that providing addr_nxt as an output would allow decoding the
// next address...   Those decodes can then be latched to provide chip-selects
// which are available at the same time as the address is available!  However,
// those decodes would not be able to be controlled by another bus master, so
// be careful about that...  


always @(posedge clk or posedge rst) begin
    if (rst) begin
        word     <= 1;
        write    <= 0;
        load_ud  <= 0;
        load_ld  <= 0;
        a        <= 0;
    end
    else begin
        if (rdy) begin
            word    <= word_nxt;
            write   <= write_nxt;  // store
            load_ud <= (ldst_nxt && word && ~write_nxt);
            load_ld <= (ldst_nxt && ~write_nxt);
            a       <= addr_nxt;
        end
    end
end

assign stb_o = rdy; // Always issue a bus cycle when possible!!!
assign  res[7:0]  = load_ld ? d[7:0]  : 8'bz;
assign  res[15:8] = load_ud ? d[15:8] : 8'bz;

assign store_udlt = 1'b0;  // Revise this for byte accesses!!!!
assign store_ld = rdy && write;
assign store_ud = rdy && write && word;

// "rdy" is included below to prevent driving the data bus when the processor
// is "halted" (halt is currently done by de-asserting rdy.)
assign  d[7:0]  = (store_ld) ? res[7:0]  : 8'bz;
assign  d[15:8] = (store_ud) ? res[15:8] : 8'bz;


endmodule
